package com.ruoyi.web.controller.pc;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.TImage;
import com.ruoyi.system.domain.TRecord;
import com.ruoyi.system.domain.req.AppRecordReq;
import com.ruoyi.system.domain.req.TRecordResultReq;
import com.ruoyi.system.domain.resp.TRecordResp;
import com.ruoyi.system.service.ITImageService;
import com.ruoyi.system.service.ITRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 诊断记录信息Controller
 *
 * @author ruoyi
 * @date 2022-08-03
 */
@Api("pc-就诊记录")
@RestController
@RequestMapping("/system/record")
public class TRecordController extends BaseController
{
    @Autowired
    private ITRecordService tRecordService;

    @Resource
    private ITImageService imageService;

    /**
     * 查询诊断记录信息列表
     */
    @ApiOperation("查询就诊记录列表")
    @GetMapping("/list")
    @Log(title = "查询就诊记录列表", businessType = BusinessType.OTHER)
    public TableDataInfo list(TRecordResultReq tRecord)
    {
        startPage();
        List<TRecordResp> list = tRecordService.selectTRecordListWithoutResult(tRecord);
        return getDataTable(list);
    }

    /**
     * 导出诊断记录信息列表
     */
    @Log(title = "诊断记录信息导出", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody @Validated TRecordResultReq tRecord)
    {
//        startPage();
        List<TRecordResp> list = tRecordService.selectTRecordListWithoutResult(tRecord);
        if(CollectionUtils.isNotEmpty(list)){
            list.stream().forEach(tRecordResp -> {
                StringBuilder updateResult = new StringBuilder();
                if(StringUtils.isNotEmpty(tRecordResp.getUpdateResult())
                        && null != JSON.parseObject(tRecordResp.getUpdateResult())){
                    typeAppend(tRecordResp.getUpdateResult(),updateResult);
                }
                if(updateResult.length() != 0){
                    tRecordResp.setUpdateResult(updateResult.toString());
                }
                StringBuilder aiResult = new StringBuilder();
                if(StringUtils.isNotEmpty(tRecordResp.getAiResult())
                        && null != JSON.parseObject(tRecordResp.getAiResult())){
                    typeAppend(tRecordResp.getAiResult(),aiResult);
                }
                if(aiResult.length() != 0){
                    tRecordResp.setAiResult(aiResult.toString());
                }
            });
        }
        ExcelUtil<TRecordResp> util = new ExcelUtil<>(TRecordResp.class);
        util.exportExcel(response, list, "诊断记录信息数据");
    }

    /**
     * 获取诊断记录信息详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:record:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "获取诊断记录信息详细信息", businessType = BusinessType.OTHER)
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tRecordService.selectTRecordById(id));
    }

    /**
     * 新增诊断记录信息
     */
    @PreAuthorize("@ss.hasPermi('system:record:add')")
    @Log(title = "新增诊断记录信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TRecord tRecord)
    {
        return toAjax(tRecordService.insertTRecord(tRecord));
    }

    /**
     * 修改诊断记录信息
     */
    @PreAuthorize("@ss.hasPermi('system:record:edit')")
    @Log(title = "修改诊断记录信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TRecord tRecord)
    {
        return toAjax(tRecordService.updateTRecord(tRecord));
    }

    /**
     * 删除诊断记录信息
     */
    @PreAuthorize("@ss.hasPermi('system:record:remove')")
    @Log(title = "删除诊断记录信息", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tRecordService.deleteTRecordByIds(ids));
    }

    @GetMapping("getInfo/{id}")
    @ApiOperation("查询患者诊断记录信息")
    @Log(title = "查询患者诊断记录信息", businessType = BusinessType.OTHER)
    public AjaxResult getInfoById(@PathVariable("id") Long id){
        if(id == null){
            return AjaxResult.error("未获取到id");
        }
        Map<String,Object> data = new HashMap<>();
        TRecordResp tRecordResp = tRecordService.selectById(id);
        data.put("result", tRecordResp);
        TImage tImage = new TImage();
        tImage.setThirdId(id);
        List<TImage> images = imageService.selectTImageList(tImage);
        data.put("image", images);
        return AjaxResult.success(data);
    }

    @GetMapping("/image/{recordId}")
    @ApiOperation("获取图片接口")
    @Log(title = "获取图片接口", businessType = BusinessType.OTHER)
    public AjaxResult getAiImage(@PathVariable("recordId") Long recordId){
        if(recordId == null){
            return AjaxResult.error("未获取到参数");
        }
        TImage tImage = new TImage();
        tImage.setThirdId(recordId);
        List<TImage> images = imageService.selectTImageList(tImage);
        return AjaxResult.success(images);
    }

    @GetMapping("getHistoryRecordList")
    @ApiOperation("查询患者历史数据图表")
    @Log(title = "查询患者历史数据图表", businessType = BusinessType.OTHER)
    public AjaxResult getHistoryRecordList(TRecord tRecord){
        return tRecordService.getHistoryList(tRecord);
    }


    /**
     * 诊断记录上传
     *
     * @return {@code AjaxResult}
     */
    @ApiOperation("诊断记录上传")
    @PostMapping("diagnostic/upload")
    @Log(title = "诊断记录上传", businessType = BusinessType.UPDATE)
    public AjaxResult diagnosticRecordsUploaded(@RequestBody AppRecordReq appRecordReq) {
        TRecord tRecord = new TRecord();
        BeanUtils.copyProperties(appRecordReq, tRecord);
        return AjaxResult.success(tRecordService.updateTRecord(tRecord));
    }

    public void typeAppend(String type,StringBuilder resultBuilder){
        //获取json字符data中的数据
        String dataString = JSON.parseObject(type).getString("data");
        try{
            if(StringUtils.isNotEmpty(dataString) && !JSON.parseObject(dataString,Map.class).isEmpty()){
                Map<String,JSONObject> mapData = JSON.parseObject(dataString,Map.class);
                //如果包含舌质
                if(mapData.containsKey("shezhi")
                        && null != mapData.get("shezhi")){
                    resultBuilder.append("舌质");
                    resultAppend(mapData.get("shezhi").toString(),resultBuilder);
                    JSONObject shezhiObject = mapData.get("shezhi");
                    if(StringUtils.isNotEmpty(shezhiObject.getString("sz_color"))
                            && null != JSON.parseObject(shezhiObject.getString("sz_color"))){
                        resultAppend(shezhiObject.getString("sz_color"),resultBuilder);
                    }
                    if(StringUtils.isNotEmpty(shezhiObject.getString("sz_yuban"))
                            && null != JSON.parseObject(shezhiObject.getString("sz_yuban"))){
                        resultAppend(shezhiObject.getString("sz_yuban"),resultBuilder);
                    }
                    if(StringUtils.isNotEmpty(shezhiObject.getString("sz_yudian"))
                            && null != JSON.parseObject(shezhiObject.getString("sz_yudian"))){
                        resultAppend(shezhiObject.getString("sz_yudian"),resultBuilder);
                    }
                    if(StringUtils.isNotEmpty(shezhiObject.getString("sz_liewen"))
                            && null != JSON.parseObject(shezhiObject.getString("sz_liewen"))){
                        resultAppend(shezhiObject.getString("sz_liewen"),resultBuilder);
                    }
                }
                //如果包含舌苔
                if(mapData.containsKey("shetai")
                        && null != mapData.get("shetai")){
                    resultBuilder.append("\n\n舌苔");
                    JSONObject shetaiObject = mapData.get("shetai");
                    if(StringUtils.isNotEmpty(shetaiObject.getString("st_color"))
                            && null != JSON.parseObject(shetaiObject.getString("st_color"))){
                        resultAppend(shetaiObject.getString("st_color"),resultBuilder);
                    }
                    resultAppend(mapData.get("shetai").toString(),resultBuilder);
                }
                //如果包含舌下脉络
                if(mapData.containsKey("shexia")
                        && null != mapData.get("shexia")){
                    resultBuilder.append("\n\n舌下脉络");
                    JSONObject shexiaObject = mapData.get("shexia");
                    if(StringUtils.isNotEmpty(shexiaObject.getString("sx_vein_color"))
                            && shexiaObject.getString("sx_vein_color").contains("{")
                            && null != JSON.parseObject(shexiaObject.getString("sx_vein_color"))){
                        resultAppend(shexiaObject.getString("sx_vein_color"),resultBuilder);
                    }
                    resultAppend(mapData.get("shexia").toString(),resultBuilder);
                }
            }
        }catch (Exception e){
            logger.error("方法：/system/record/export报错，json字符串："+type,e.getMessage());
        }
    }

    /**
     * 组装诊断结果
     * @param type
     * @param resultBuilder
     */
    public void resultAppend(String type,StringBuilder resultBuilder){
        JSONObject typeObject = JSON.parseObject(type);
        if(StringUtils.isNotEmpty(typeObject.getString("sz_tip"))
                && null != JSON.parseObject(typeObject.getString("sz_tip"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sz_tip"));
            resultBuilder.append("\n舌尖红:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sz_laonen"))
                && null != JSON.parseObject(typeObject.getString("sz_laonen"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sz_laonen"));
            resultBuilder.append("\n舌质老嫩:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sz_chihen"))
                && null != JSON.parseObject(typeObject.getString("sz_chihen"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sz_chihen"));
            resultBuilder.append("\n舌质齿痕:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sz_color2"))
                && null != JSON.parseObject(typeObject.getString("sz_color2"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sz_color2"));
            resultBuilder.append("\n舌质颜色:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("st_color2"))
                && null != JSON.parseObject(typeObject.getString("st_color2"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("st_color2"));
            resultBuilder.append("\n舌苔颜色:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sz_avg_red"))
                && null != JSON.parseObject(typeObject.getString("sz_avg_red"))
                && StringUtils.isNotEmpty(JSON.parseObject(typeObject.getString("sz_avg_red")).getString("v"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sz_avg_red"));
            resultBuilder.append("\n红度平均值:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sz_avg_purple"))
                && null != JSON.parseObject(typeObject.getString("sz_avg_purple"))
                && StringUtils.isNotEmpty(JSON.parseObject(typeObject.getString("sz_avg_purple")).getString("v"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sz_avg_purple"));
            resultBuilder.append("\n紫度平均值:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sz_pangshou"))
                && null != JSON.parseObject(typeObject.getString("sz_pangshou"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sz_pangshou"));
            resultBuilder.append("\n舌质胖瘦:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sz_pangshou"))
                && null != JSON.parseObject(typeObject.getString("sz_pangshou"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sz_pangshou"));
            resultBuilder.append("\n胖度值:"+(StringUtils.isNotEmpty(valObject.getString("quantification")) ? valObject.getString("quantification") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sz_yuban_area"))
                && null != JSON.parseObject(typeObject.getString("sz_yuban_area"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sz_yuban_area"));
            resultBuilder.append("\n瘀斑面积:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sz_yudian_area"))
                && null != JSON.parseObject(typeObject.getString("sz_yudian_area"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sz_yudian_area"));
            resultBuilder.append("\n瘀点面积:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sz_yudian_num"))
                && null != JSON.parseObject(typeObject.getString("sz_yudian_num"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sz_yudian_num"));
            resultBuilder.append("\n瘀点个数:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sz_is_liewen"))
                && null != JSON.parseObject(typeObject.getString("sz_is_liewen"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sz_is_liewen"));
            resultBuilder.append("\n舌质裂纹:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("st_boluo"))
                && null != JSON.parseObject(typeObject.getString("st_boluo"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("st_boluo"));
            resultBuilder.append("\n剥落苔:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("st_area"))
                && null != JSON.parseObject(typeObject.getString("st_area"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("st_area"));
            resultBuilder.append("\n舌苔面积:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("st_houbo"))
                && null != JSON.parseObject(typeObject.getString("st_houbo"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("st_houbo"));
            resultBuilder.append("\n舌苔厚薄:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("st_houbo"))
                && null != JSON.parseObject(typeObject.getString("st_houbo"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("st_houbo"));
            resultBuilder.append("\n厚度值:"+(StringUtils.isNotEmpty(valObject.getString("quantification")) ? valObject.getString("quantification") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("st_funi"))
                && null != JSON.parseObject(typeObject.getString("st_funi"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("st_funi"));
            resultBuilder.append("\n舌苔腐腻:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("st_funi"))
                && null != JSON.parseObject(typeObject.getString("st_funi"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("st_funi"));
            resultBuilder.append("\n腐腻值:"+(StringUtils.isNotEmpty(valObject.getString("quantification")) ? valObject.getString("quantification") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("st_runzao"))
                && null != JSON.parseObject(typeObject.getString("st_runzao"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("st_runzao"));
            resultBuilder.append("\n舌苔润燥:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("st_runzao"))
                && null != JSON.parseObject(typeObject.getString("st_runzao"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("st_runzao"));
            resultBuilder.append("\n润度值:"+(StringUtils.isNotEmpty(valObject.getString("quantification")) ? valObject.getString("quantification") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("st_cao"))
                && null != JSON.parseObject(typeObject.getString("st_cao"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("st_cao"));
            resultBuilder.append("\n糙苔:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("st_cao"))
                && null != JSON.parseObject(typeObject.getString("st_cao"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("st_cao"));
            resultBuilder.append("\n糙度值:"+(StringUtils.isNotEmpty(valObject.getString("quantification")) ? valObject.getString("quantification") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("st_avg_yellow"))
                && null != JSON.parseObject(typeObject.getString("st_avg_yellow"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("st_avg_yellow"));
            resultBuilder.append("\n黄度平均值:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sx_vein_width"))
                && typeObject.getString("sx_vein_width").contains("{")
                && null != JSON.parseObject(typeObject.getString("sx_vein_width"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sx_vein_width"));
            resultBuilder.append("\n脉络粗细:"+(StringUtils.isNotEmpty(valObject.getString("quantification")) ? valObject.getString("quantification") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sx_vein_length"))
                && typeObject.getString("sx_vein_length").contains("{")
                && null != JSON.parseObject(typeObject.getString("sx_vein_length"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sx_vein_length"));
            resultBuilder.append("\n脉络长度:"+(StringUtils.isNotEmpty(valObject.getString("quantification")) ? valObject.getString("quantification") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sx_vein_type"))
                && typeObject.getString("sx_vein_type").contains("{")
                && null != JSON.parseObject(typeObject.getString("sx_vein_type"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sx_vein_type"));
            resultBuilder.append("\n脉络形态:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sx_vein_color2"))
                && null != JSON.parseObject(typeObject.getString("sx_vein_color2"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sx_vein_color2"));
            resultBuilder.append("\n脉络颜色:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
        if(StringUtils.isNotEmpty(typeObject.getString("sx_vein_avg_purple"))
                && null != JSON.parseObject(typeObject.getString("sx_vein_avg_purple"))){
            JSONObject valObject = JSON.parseObject(typeObject.getString("sx_vein_avg_purple"));
            resultBuilder.append("\n紫度平均值:"+(StringUtils.isNotEmpty(valObject.getString("v")) ? valObject.getString("v") : ""));
        }
    }
}
